#     file     : python/t_Configuration.py
#     author   : Marcus Rickert
#     RCS      : $Id: t_Configuration.py,v 1.4 2005/03/29 20:37:27 mr Exp $
#     purpose  : class t_Configuration
#
#     Copyright (C) 2005 :  Marcus Rickert

# TODO:
#

import os
import re
import sys
import string
import types

from t_AppException import *

C_DEFAULT_GROUP = "DEFAULT"

class t_Configuration:

    def __init__(self, p_Filename, p_AllowAnyGroupNames=0, p_AllowAnyItemValues=0):

        self.f_AllowAnyGroupNames = p_AllowAnyGroupNames
        self.f_AllowAnyItemValues = p_AllowAnyItemValues
        try:
            v_File = open(p_Filename, 'r')
            
        except IOError, v_IOError:
            raise t_AppException(p_Class = self, p_Message="while opening configuration file for reading",
                                 p_Filename = p_Filename, p_IOError = v_IOError)
        try:
            v_RE_Comment = re.compile(r"^\s*#|^\s*$")
            if self.f_AllowAnyGroupNames:                
                v_RE_Group = re.compile(r"^\s*\[(.+)\]\s*$")
            else:
                v_RE_Group = re.compile(r"^\s*\[(\S+)\]\s*$")
            if self.f_AllowAnyItemValues:
                v_RE_Item = re.compile(r"^\s*(\w+)\s*(\+?)=\s*([^;]*)\s*;?.*$")
                v_RE_ShortItem = re.compile(r"^\s*([^;]+)\s*;?\s*$")
            else:
                v_RE_Item = re.compile(r"^\s*(\w+)\s*(\+?)=\s*([-a-zA-Z_0-9.@:%/\"]+)\s*;?\s*$")
                v_RE_ShortItem = re.compile(r"^\s*(\w+)\s*;?\s*$")
            v_Group = C_DEFAULT_GROUP
            self.f_Settings = { }
            self.f_Groups = { v_Group : {} }
            self.f_SortedGroups = [ ]
            v_LineNumber = 0
            for v_Line in v_File.readlines():
                v_LineNumber = v_LineNumber + 1
                if v_RE_Comment.match(v_Line):
                    continue
                v_MO_Group = v_RE_Group.match(v_Line)
                if v_MO_Group:
                    if self.f_AllowAnyGroupNames:
                        v_Group = v_MO_Group.group(1);
                    else:
                        v_Group = string.upper(v_MO_Group.group(1));
                    if not self.f_Groups.has_key(v_Group):
                        self.f_Groups[v_Group] = { }
                        self.f_SortedGroups.append(v_Group)
                else:
                    v_MO_Item = v_RE_Item.match(v_Line)
                    if v_MO_Item:
                        v_Item = string.upper(v_MO_Item.group(1))
                        v_Key = v_Group + '.' + v_Item
                        v_Value = v_MO_Item.group(3)
                        if v_MO_Item.group(2):
                            if not self.f_Settings.has_key(v_Key):
                                self.f_Settings[v_Key] = []
                            self.f_Settings[v_Key].append(v_Value)
                            if not self.f_Groups[v_Group].has_key(v_Item):
                                self.f_Groups[v_Group][v_Item] = [];
                            self.f_Groups[v_Group][v_Item].append(v_Value);
                        else:
                            self.f_Settings[v_Key] = v_Value
                            self.f_Groups[v_Group][v_Item] = v_Value;
                    else:
                        v_MO_ShortItem = v_RE_ShortItem.match(v_Line)
                        if v_MO_ShortItem:
                            v_Item = string.upper(v_MO_ShortItem.group(1))
                            v_Key = v_Group + '.' + v_Item
                            v_Value = 1
                            self.f_Settings[v_Key] = v_Value
                            self.f_Groups[v_Group][v_Item] = v_Value;                            
                            
                        else:
                            sys.stderr.write("WARNING: %s: invalid line '%s' in file '%s' line %d\n" % \
                                             (sys.argv[0], v_Line[:-1], p_Filename, v_LineNumber))
            
        except IOError, v_IOError:
            raise t_AppException(p_Message="while reading configuration file '%s'" % p_Filename,
                                 p_IOError=v_IOError);
        
        v_File.close();
            
        
    def findInGroup(self, p_GroupName, p_ItemName, p_Default = None, p_Unquote = 1):

        
        v_Key = string.upper(p_GroupName + "." + p_ItemName);
        if self.f_Settings.has_key(v_Key):
            v_Result = self.f_Settings[v_Key]

        elif p_Default != None:
            v_Result = p_Default
            
        else:
            v_DefaultKey = string.upper(C_DEFAULT_GROUP + "." + p_ItemName);
            print "check default key ", v_DefaultKey
            if self.f_Settings.has_key(v_DefaultKey):
                print "return default ", self.f_Settings[v_DefaultKey]
                v_Result = self.f_Settings[v_DefaultKey]
            else:
                raise t_AppException(p_Class=self,
                                     p_Message="cannot find configuration group.item '%s'" % v_Key)
            
        if p_Unquote and type(v_Result) == types.StringType and v_Result[0] == "\"" and v_Result[-1] == "\"":
            v_Result = v_Result[1:-1]
        return v_Result

    def setInGroup(self, p_GroupName, p_ItemName, p_Value = 1):

        v_Group = string.upper(p_GroupName)
        v_Item = string.upper(p_ItemName)
        v_Key = v_Group + "." + v_Item
        
        self.f_Settings[v_Key] = p_Value
        self.f_Groups[v_Group][v_Item] = p_Value
        
    def writeToFile(self, p_Filename, p_Terminator=""):
        try:
            v_File = open(p_Filename, 'w')
            
        except IOError, v_IOError:
            raise t_AppException(p_Class = self, p_Message="while opening configuration file for writing",
                                 p_Filename = p_Filename, p_IOError = v_IOError)

        try:
            for v_Group in self.f_Groups.items():

                v_GroupName = v_Group[0]

                v_File.write("[%s]\n" % v_GroupName)
                
                v_Items = v_Group[1]

                for v_Item in v_Items.items():

                    v_ItemName = v_Item[0]
                    v_Value = v_Item[1]
                    v_File.write("%s=%s%s\n" % (v_ItemName, v_Value, p_Terminator));
                                    
        except IOError, v_IOError:
            raise t_AppException(p_Message="while reading configuration file '%s'" % p_Filename,
                                 p_IOError=v_IOError);
        
        v_File.close();
