/*      file     : RouteConverter/TLinkHitStat.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TLinkHitStat.C,v 1.2 2005/03/25 12:28:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <math.h>

#include <TBaseStream.h>
#include <TTimeStep.h>
#include <TEdge.h>

#include "Transims.h"
#include "TLinkHitStat.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TLinkHitStat::ClassInfo = 0;

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#  include <TLinkHitStat.i.C>
#endif

/* =================================================================================== */
#define ParentClass TParallelObject
/* =================================================================================== */

TLinkHitStat::TLinkHitStat(TParallelID theParallelID, long theStartTimeStep,
                           long theEndTimeStep, long theBinSize) :

ParentClass(ClassInfo, theParallelID)

{
    int CBin;

    Reset();
    StartTimeStep = theStartTimeStep;
    BinSize = theBinSize;
    NrOfBins = (theEndTimeStep - theStartTimeStep) / theBinSize + 1;
    Hits = new long [NrOfBins];
    for (CBin = 0; CBin < NrOfBins; CBin++)
        Hits[CBin] = 0;
}

/* =================================================================================== */

TLinkHitStat::~TLinkHitStat(void)

{
    DeleteObjectArray(Hits);
}

/* =================================================================================== */

TString
TLinkHitStat::GetClassName(void) const

{
    return "TLinkHitStat";
}

/* =================================================================================== */

void
TLinkHitStat::Reset(void)

{
    StartTimeStep = 0;
    BinSize = 1;
    Hits = 0;
}

/* =================================================================================== */

bool
TLinkHitStat::Write(TBaseStream *Stream)

{
    TEdge       *Edge;
    long         TransimsLinkID;
    TParallelID  FromNodeID;
    long         CBin;

    if ((Edge = (TEdge *) TEdge::ClassInfo->GetObjectPtr(GetParallelID())) == 0)
    {
        Error("WriteTransimsLinkStat: cannot retrieve edge for %ld", GetParallelID());
        return FALSE;
    }
    FromNodeID = Edge->GetFromNodeID();
    TransimsLinkID = PaminaEdge2TransimsLink(GetParallelID());
    for (CBin = 0; CBin < NrOfBins; CBin++)
        if (Hits[CBin])
            Stream->Write("%ld %ld %ld %ld\n", (long) TransimsLinkID, (long) FromNodeID, 
                          (long) (StartTimeStep + CBin * BinSize), (long) Hits[CBin]);
    return TRUE;
}

/* =================================================================================== */
#undef ParentClass
