/*      file     : TrafficData/TCAManager.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TCAManager.h,v 1.12 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCAMANAGER_H
#define TCAMANAGER_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <TStatHandler.h>

// ***FORWARDS************************************************************************

class TDistributedRange;
class TMultilaneGrid;
class TMultilaneGridConfig;
class TMultilaneStat;
class TSamplingDescription;
class TVehicle;

// ***CLASSES************************************************************************* 

class TCAManager : public TBaseObject, public TObjectHandler, public TStatHandler

{
public:

    TCAManager(TEdgeList *theEdges, TEdgeList *theBoundaryEdges, 
               TEdgeList *theNonBoundaryEdges);
    virtual ~TCAManager(void);

    bool
    Activate(void);

    bool
    ExecuteAccessories(const TTimeStep &TimeStep  /*, TEdge *Edge */);

    bool
    ExecuteSubTimeStep(TTimeStep &TimeStep, bool BoundariesAvailable);

    bool
    InitializeEdgeAfterReception(TEdge *Edge, TDistributedRange *ActiveRange);

    bool
    Run(long theStartTimeStep);
    
private:

    virtual bool
    Collect(TSamplingDescription *Description);

    bool
    CollectGridStat(const TTimeStep &TimeStep, TStatObjectList *StatList);

    bool
    CollectMultilaneDump(TTimeStep &TimeStep, const TRectangle &Frame, 
                         TStatObjectList *List);

    bool
    CollectMultilaneStat(TMultilaneStat *Stat);

    virtual TString
    GetClassName(void) const;

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);

    virtual bool
    HandleLoadRequest(TParallelObject *Object, bool &Loaded);

    bool
    LoadMultilaneGrid(TMultilaneGrid *Grid);

    bool
    RandomFill(TMultilaneGrid *Grid);

    virtual bool
    Sample(TSamplingDescription *Description);

    bool
    SampleGridStat(const TTimeStep &TimeStep);

    TMultilaneGridConfig *Default;

    long                  StartTimeStep;
    double                InitDensity;    
    TEdgeList            *Edges;
    TEdgeList            *BoundaryEdges;
    TEdgeList            *NonBoundaryEdges;
    long                  StartDump;
    long                  StopDump;
    long                  DumpWidth;
    bool                  BeforeFirstTimeStep;
    long                  MinSites;
    double                DecProb;
    bool                  SpeedLimits;
    bool                  QueueFeedback;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

