/*      file     : TrafficData/TIntersection.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TIntersection.h,v 1.6 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TINTERSECTION_H
#define TINTERSECTION_H

// ***INCLUDES************************************************************************ 

#include <TParallelObject.h>

// ***FORWARDS************************************************************************

class TBaseStream;
class TMultilaneGrid;
class TIncidenceInfo;
class TIntersectionControl;
class TTransferInfo;
class TTimeStep;
class TSource;
class TSink;
class TVehicle;

// ***DEFINES************************************************************************* 

#define I_FLAG_MINOR_LINK    0
#define I_FLAG_MAJOR_LINK    1
#define I_FLAG_LINK_MASK     1

// ***CLASSES************************************************************************* 

class TIntersectionVarData

{
public:

    TIntersectionVarData(void);

    long RoutesFailed;
};

// ===================================================================================

class TIntersection : public TParallelObject

{
public:
  
    TIntersection(TClassInfo *ClassInfo, TParallelID theParallelID);
    virtual ~TIntersection(void);

    virtual bool
    Activate(void);

    bool
    AddVehicle(TVehicle *Vehicle);

    virtual bool
    AssignEdges(TIncidenceInfo *theIncidenceInfo);

    virtual bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    inline TIncidenceInfo *
    GetIncidenceInfo(void);

    inline TMultilaneGrid *
    GetIncomingGrid(int Nr) const;

    TMultilaneGrid *
    GetOutgoingGridByID(TParallelID EdgeID) const;

    inline TMultilaneGrid *
    GetOutgoingGrid(int Nr) const;

    int
    GetOutgoingGridNrByID(TParallelID EdgeID) const;

    TSource *
    GetSource(TParallelID GridID) const;

    long 
    GetTurnCount(int FromGridNr, int ToGridNr) const;

    inline bool
    IsIncomingSegmentActive(long EdgeNr);

    inline bool
    IsIncomingVehicleActive(long EdgeNr);

    void
    ResetTurnCounts(void);

    void
    VehicleFailed(TVehicle *Vehicle);

protected:

    bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;
    
    bool
    Turn(int FromGridNr, int ToGridNr);

    TIncidenceInfo        *IncidenceInfo;
    TMultilaneGrid       **IncomingGrids;
    TMultilaneGrid       **OutgoingGrids;
    TSource              **Sources;
    TSink                 *Sink;
    long                  *TurnCounts;

private:

    TIntersectionVarData   Var;

    // temp:

    TIntersectionControl *IntersectionControl;
};


// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TIntersection.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
