/*      file     : TrafficData/TSource.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TSource.h,v 1.5 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSOURCE_H
#define TSOURCE_H

// ***INCLUDES************************************************************************ 

#include <TParallelObject.h>

// ***FORWARDS************************************************************************

class TClassInfo;
class TMultilaneGrid;
class TRoutingStat;
class TTimeStep;
#include "TVehicle.forward.h"

// ***CLASSES************************************************************************* 

class TSourceVarData 

{
public:

    TSourceVarData(void);
    ~TSourceVarData(void);

    TVehicleList  *PendingVehicles;
    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define TSOURCE_LONG 3
    long           NormalInsertions;
    long           DelayedInsertions;
    long           SumDelay;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ===================================================================================

class TSource : public TParallelObject

{
    friend class TRoutingStat;

public:

    TSource(TParallelID theParallelID);
    virtual ~TSource(void);

    bool
    AddVehicle(TVehicle *Vehicle);

    bool
    AssignGrid(TMultilaneGrid *theGrid, long theOffset);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    bool
    ExecuteSubTimeStep(const TTimeStep &TimeStep);

    double
    GetAverageDelay(void) const;

    virtual TString
    GetInstanceName(void) const;

    static TClassInfo *ClassInfo;
    static int         InsertionGap;
    static int         InsertionSlack;
    static long        StartTimeStep;

private:

    bool
    DecodeVehicles(TBaseStream *aStream);

    bool
    EncodeVehicles(TBaseStream *aStream) const;

    virtual TString
    GetClassName(void) const;

    TSourceVarData  Var;
    TMultilaneGrid *Grid;
    long            Offset;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
