/*      file     : TrafficData/TTurnCountStat.C
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TTurnCountStat.C,v 1.4 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TBaseStream.h>
#include <TTimeStep.h>
#include <TStream.h>

#include <Transims.h>
#include <TIncidenceInfo.h>
#include <TIntersection.h>
#include <TMultilaneGrid.h>
#include "TTurnCountStat.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TTurnCountStat::ClassInfo = 0;

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TTurnCountStat::TTurnCountStat(TParallelID theParallelID) : 

ParentClass(ClassInfo, theParallelID)

{
    Reset();
}

// =================================================================================== 

TTurnCountStat::~TTurnCountStat(void)

{
    DeleteObjectArray(IncomingEdgeID);
    DeleteObjectArray(OutgoingEdgeID);
    DeleteObjectArray(TurnCounts);
}

// =================================================================================== 

bool
TTurnCountStat::Allocate(void)

{
    IncomingEdgeID = new TParallelID [NrOfIncomingEdges];
    OutgoingEdgeID = new TParallelID [NrOfOutgoingEdges];
    TurnCounts = new long [NrOfIncomingEdges * NrOfOutgoingEdges];
    return TRUE;
}

// =================================================================================== 

bool
TTurnCountStat::Collect(TIntersection *Intersection)

{
    int CEdge, CIncoming, COutgoing;

    NrOfIncomingEdges = Intersection->GetIncidenceInfo()->GetNrOfIncomingEdges();
    NrOfOutgoingEdges = Intersection->GetIncidenceInfo()->GetNrOfOutgoingEdges();
    if (!Allocate())
        return FALSE;
    for (CEdge = 0; CEdge < NrOfIncomingEdges; CEdge++)
        IncomingEdgeID[CEdge] = Intersection->GetIncomingGrid(CEdge)->GetParallelID();
    for (CEdge = 0; CEdge < NrOfOutgoingEdges; CEdge++)
        OutgoingEdgeID[CEdge] = Intersection->GetOutgoingGrid(CEdge)->GetParallelID();
    for (CIncoming = 0; CIncoming < NrOfIncomingEdges; CIncoming++)
        for (COutgoing = 0; COutgoing < NrOfOutgoingEdges; COutgoing++)
            TurnCounts[CIncoming * NrOfOutgoingEdges + COutgoing] = 
                   Intersection->GetTurnCount(CIncoming, COutgoing);
    return TRUE;
}

// =================================================================================== 

bool
TTurnCountStat::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    if (!(ParentClass::Decode(aStream, TransferInfo) AND
          aStream->Decode(NrOfIncomingEdges) AND
          aStream->Decode(NrOfOutgoingEdges)))
        return FALSE;
    if (!Allocate())
        return FALSE;
    if (!(aStream->Decode(IncomingEdgeID[0], NrOfIncomingEdges) AND
          aStream->Decode(OutgoingEdgeID[0], NrOfOutgoingEdges) AND
          aStream->Decode(TurnCounts[0], NrOfIncomingEdges * NrOfOutgoingEdges)))
        return FALSE;
    return TRUE;
}

// =================================================================================== 

bool
TTurnCountStat::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(aStream, TransferInfo) AND
            aStream->Encode(NrOfIncomingEdges) AND
            aStream->Encode(NrOfOutgoingEdges) AND
            aStream->Encode(IncomingEdgeID[0], NrOfIncomingEdges) AND
            aStream->Encode(OutgoingEdgeID[0], NrOfOutgoingEdges) AND
            aStream->Encode(TurnCounts[0], NrOfIncomingEdges * NrOfOutgoingEdges));
}

// =================================================================================== 

void
TTurnCountStat::Reset(void)

{
    NrOfIncomingEdges = 0;
    NrOfOutgoingEdges = 0;
    IncomingEdgeID = 0;
    OutgoingEdgeID = 0;
    TurnCounts = 0;
}

// =================================================================================== 

bool
TTurnCountStat::Write(TStream *Stream)

{
    int  CIncoming, COutgoing;
    long Count;

    for (CIncoming = 0; CIncoming < NrOfIncomingEdges; CIncoming++)
        for (COutgoing = 0; COutgoing < NrOfOutgoingEdges; COutgoing++)
            if ((Count = TurnCounts[CIncoming * NrOfOutgoingEdges + COutgoing]) > 0)
                Stream->Write("%ld %ld %ld %ld %ld\n",
                              (long) GetTimeStep(),
                              (long) GetParallelID(),
                              (long) PaminaEdge2TransimsLink(IncomingEdgeID[CIncoming]),
                              (long) PaminaEdge2TransimsLink(OutgoingEdgeID[COutgoing]),
                              (long) Count);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
