/*      file     : common/TBaseStream.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TBaseStream.C,v 1.9 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <string.h>
#include <stdlib.h>
#include <string.h>

#include <Error.h>
#include <TDebug.h>

#include <TBaseStream.h>

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

#include "TBaseStream.Bodies.C"

// =================================================================================== 

bool
TBaseStream::DecodeStr(char **aRawString)

{
    long Length;

    if (!Decode(Length))
    {
        LocalErrorStatus = Status_CommunicationError;
        Error("cannot decode string length");
        return FALSE;
    }
    if ((*aRawString = (char *) malloc(Length + 1)) == 0)
    {
        LocalErrorStatus = Status_OutOfHeap;
        return FALSE;
    }
    (*aRawString)[Length] = 0;
    return Decode((signed char &) **aRawString, Length);
}

// =================================================================================== 

bool
TBaseStream::DecodeStr(char *aRawString)

{
    long Length;

    if (aRawString == 0)
    {
        Error("DecodeStr: NULL string");
        return FALSE;
    }
    if (!Decode(Length))
    {
        LocalErrorStatus = Status_CommunicationError;
        Error("cannot decode string length");
        return FALSE;
    }
    aRawString[Length] = 0;
    return Decode((signed char &) *aRawString, Length);
}

/* =================================================================================== */

// This won't work on heterongeous architectures with mixed small and big Endian!

bool
TBaseStream::Decode(LongID &Data, long Count, long Stride)

{
    return Decode((long &) Data, 2*Count, Stride);
}

/* =================================================================================== */

// This won't work on heterongeous architectures with mixed small and big Endian!

bool
TBaseStream::Encode(const LongID &Data, long Count, long Stride) const

{
    return Encode((long &) Data, 2*Count, Stride);
}

/* =================================================================================== */
#ifndef NO_BOOL
bool
TBaseStream::DecodeBool(bool &Data, long Count, long Stride)

{
    return Decode((unsigned int &) Data, Count, Stride);
}
#endif
/* =================================================================================== */
#ifndef NO_BOOL
bool
TBaseStream::EncodeBool(const bool &Data, long Count, long Stride) const

{
    return Encode((unsigned int &) Data, Count, Stride);
}
#endif
// =================================================================================== 

bool
TBaseStream::EncodeStr(const char *aRawString) const

{
    long Length;

#ifdef CHECK_STRING
    if (aRawString == 0)
    {
        Error("Encode: null string");
        return FALSE;
    }
#endif
    Length = (long) strlen(aRawString);
    return (Encode(Length) AND
            Encode((signed char &) *aRawString, Length));
}

// =================================================================================== 

bool
TBaseStream::Event(TEventID theEventID) const

{
    debug(Print(DM_EVENT, "encode event with ID %ld", theEventID));
    return Encode(theEventID);
}

// =================================================================================== 

bool
TBaseStream::IsBinary(void) const

{
    return TRUE;
}

// =================================================================================== 

bool
TBaseStream::Write(const char *, ...)

{
    NotOverloaded("Write");
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
