/*      file     : common/TConfiguration.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TConfiguration.h,v 1.11 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TCONFIGURATION_H
#define TCONFIGURATION_H

// ***INCLUDES************************************************************************ 

#include "TString.h"
#include "TStringContainerEntry.h"
#include "TABTree.h"
#include "TBaseObject.h"

// ***FORWARDS************************************************************************

class TConfiguration;
class TStream;

// ***DEFINES************************************************************************* 

#define CONFIG_TABLE_STRCMP STRCMP_IGNORE_CASE

#define DEFAULT_CONFIG_VALUE "<UNKNOWN>"

// ***CLASSES************************************************************************* 

class TConfigEntry : public TBaseObject

{
    friend class TConfiguration;
    
public:

    TConfigEntry(const char *aGroup, const char *aName, 
                 const char *aValue = DEFAULT_CONFIG_VALUE);
    TConfigEntry(const char *aGroup, const char *aName, long aValue);
    TConfigEntry(const char *aGroup, const char *aName, double aValue);
    TConfigEntry(const TConfigEntry &anEntry);
    TConfigEntry(TBaseStream &aStream);

    bool
        Decode(TBaseStream &aStream);
    bool
        Encode(TBaseStream &aStream) const;
    virtual TString
        GetClassName(void) const;
    inline TString
        GetKey(void) const;
    virtual TString
        GetInstanceName(void) const;
    inline const char *
        GetValue(void) const;
    inline void
        SetDefault(const char *aValue);
    inline void
        SetValue(const char *aValue);

    bool
    WriteToFile(FILE *output);

private:

    TString Groupname;
    TString Name;
    TString Default;
    TString Value;
};

typedef TABTree < TStringContainerEntry < TConfigEntry > > TConfigTable;

// =================================================================================== 

/** Class handling configuration files.
  */

class TConfiguration : public TBaseObject

{
public:

    TConfiguration(void);
    virtual ~TConfiguration(void);

    bool
        Decode(TBaseStream &aStream);
    bool
        Encode(TBaseStream &aStream) const;
    inline bool
        Exists(const char *Groupname, const char *Name);
    virtual TString
        GetClassName(void) const;
    inline double
        GetDouble(const char *Groupname = 0, const char *Name = 0);
    inline long
        GetLong(const char *Groupname = 0, const char *Name = 0);
    const char *
        GetString(const char *Groupname = 0, const char *Name = 0);
    bool
        OptionActive(const char *Option);
    int
        ParseError(const char *ErrorString, int LineNumber);
    bool
        ReadFile(const char *Filename);
    void
        Set(const TConfigEntry &anEntry);
    void
        SetDefault(const TConfigEntry &anEntry);

    bool
    WriteToFile(FILE *output);

private:

    TConfigTable Table;
    TConfigEntry PreviousEntry;
};

// ***EXTERNALS*********************************************************************** 

extern TConfiguration *Configuration;

// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#  include "TConfiguration.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
