/*      file     : common/TDLList.C
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TDLNodePrimitive
                   TDLListPrimitive
                   TDLListReference
                   TDLEnumPrimitive
        RCS      : $Id: TDLList.C,v 1.13 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#include <stdlib.h>

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass
// =================================================================================== 

TDLNodePrimitive::TDLNodePrimitive(void) :

Refs(1)  /* set increment to 1 */

{
    FirstFreeIndex = -1;    
}

// =================================================================================== 

TDLNodePrimitive::~TDLNodePrimitive(void)

{
    UnlinkAll();
}

/* =================================================================================== */

void
TDLNodePrimitive::FreeReference(int Index) const

{
    SetList(Index, 0);
    SetNext(Index, 0, FirstFreeIndex);
    MutableAccess(TDLNodePrimitive, FirstFreeIndex) = Index;
}

/* =================================================================================== */

const TListNodePrimitive *
TDLNodePrimitive::GetNext(int Index) const

{
    return Refs[(long) Index].Next;
}

/* =================================================================================== */
 
int
TDLNodePrimitive::GetNextIndex(int Index) const

{
    return Refs[(long) Index].NextIndex;
}

/* =================================================================================== */

const TListNodePrimitive *
TDLNodePrimitive::GetPrev(int Index) const

{
    return Refs[(long) Index].Prev;
}

/* =================================================================================== */

int
TDLNodePrimitive::GetPrevIndex(int Index) const

{
    return MutableAccess(TDLNodePrimitive, Refs.Get(Index)).PrevIndex;
}

/* =================================================================================== */

bool
TDLNodePrimitive::GetFreeReferenceIndex(int &Index) const

{
    if (FirstFreeIndex == -1)
    {
        TDLListReference Reference;
    
        InitListReference(Reference);
        if (!MutableAccess(TDLNodePrimitive, Refs).Add(Reference))
        {
            Debug.Message("TDLNodePrimitive::GetFreeRefrenceIndex: cannot add reference!");
            exit(1);
        }
        Index = (int) Refs.GetNrOfItems() - 1;
    }
    else
    {
        Index = FirstFreeIndex;
        MutableAccess(TDLNodePrimitive, FirstFreeIndex) = Refs[(long) FirstFreeIndex].NextIndex;
    }
    return TRUE;    
}

/* =================================================================================== */

bool
TDLNodePrimitive::GetListIndex(const TDLListPrimitive *theList, int &Index) const

{    
    Index = 0;
    while (Index < Refs.GetNrOfItems())
        if (Refs[(long) Index].List == theList)
            return TRUE;
        else
            Index++;
    return FALSE;
}

/* =================================================================================== */

void
TDLNodePrimitive::SetList(int Index, TDLListPrimitive *theList) const

{
    MutableAccess(TDLNodePrimitive, Refs.Get(Index)).List = theList;
}

/* =================================================================================== */

void
TDLNodePrimitive::SetNext(int Index, const TListNodePrimitive *NextNode, int NextIndex) const

{
    
    MutableAccess(TDLNodePrimitive, Refs.Get(Index)).Next = NextNode;
    MutableAccess(TDLNodePrimitive, Refs.Get(Index)).NextIndex = NextIndex;
}

/* =================================================================================== */

void
TDLNodePrimitive::SetPrev(int Index, const TListNodePrimitive *PrevNode, int PrevIndex) const

{
    MutableAccess(TDLNodePrimitive, Refs.Get(Index)).Prev = PrevNode;
    MutableAccess(TDLNodePrimitive, Refs.Get(Index)).PrevIndex = PrevIndex;
}

/* =================================================================================== */

bool
TDLNodePrimitive::UnlinkAll(void)

{
    int ListNr;

    for (ListNr = 0; ListNr < Refs.GetNrOfItems(); ListNr++)
        if (Refs.Get(ListNr).List)
            if (!Unlink(Refs.Get(ListNr).List))
                return FALSE;
    return TRUE;
}


/* =================================================================================== */
#undef ParentClass
