/*      file     : common/TEnvironment.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TEnvironment.h,v 1.14 2005/03/29 20:32:32 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TENVIRONMENT_H
#define TENVIRONMENT_H

// ***INCLUDES************************************************************************ 

#include "Error.h"
#include "TBaseObject.h"

// ***FORWARDS************************************************************************

class TConfig;
class TConfiguration;
class TTimer;

// ***DEFINES************************************************************************* 

#define DEFAULT_LOCAL_ID  0

// ***TYPES*************************************************************************** 

typedef long TLocalID;

// ***CLASSES************************************************************************* 

/**
  */

class TEnvironment : public TBaseObject

{
public:

    TEnvironment(int argc, char **argv);
    virtual ~TEnvironment(void);

    inline int
        argc(void) const;
    inline char **
        argv(void) const;
    inline int *
        pargc(void);
    inline char ***
        pargv(void);
    bool
        Activate(void);
    inline TString
        GetApplicationFilename(const char *Path) const;
    inline TConfig *
        GetConfig(void) const;
    inline TConfiguration *
        GetConfiguration(void) const;
    inline const char *
        GetExecutable(void) const;
    inline TErrorStatus
        GetExitCode(void) const;
    TString
        GetFilename(const char *BaseDir, const char *Path) const;
    inline TString
        GetHomeFilename(const char *Path) const;
    inline TLocalID 
        GetLocalID(void) const;
    inline TTimer *
    GetOverallTimer(void);
    void
        InterpretConfig(void);
    virtual void
        PrintString(const char *Format ...) const;
    TString
        ReplacePathTokens(const char *Filename) const;
    inline void
        SetArchitecture(const char *theArchitecture);

    inline void
    TEnvironment::SetExitCode(TErrorStatus theExitCode);

    inline void
        SetHostName(const char *theHostName);
    inline void
        SetLocalID(TLocalID theLocalID);
    inline void
        SetPrintPrefix(const char *thePrefix);
    inline void
        SetProcessName(const char *theProcessName);
    inline void
        SetRunName(const char *theRunName);

    bool
    Run(void);

    inline void
    SetWorkDir(const char *theWorkDir);

private:

    bool
        Deactivate(void);
    virtual TString
        GetClassName(void) const;
    bool
        GetStrings(void);
    void
        _Init(void);
    bool
        InterpretOption(int c, char *OptArg, bool &Handled);
    bool
        LoadLocalConfig(void);
    bool
        ParseOptions(void);

    int      my_argc;
    char   **my_argv;

    TString         PrintPrefix;
    TString         OptionString;
    TString         Architecture;
    TString         ProcessName;
    TString         HostName;
    TString         Executable;
    TString         AppBaseDir;
    TString         HomeDir;
    TString         WorkDir;
    TString         RunName;
    TConfig        *Config;        
    TConfiguration *Configuration; 
    TLocalID        LocalID;
    bool            ThrowCore;
    bool            CheckLocalConfig;
    TTimer         *Timer;
    TTimer         *OverallTimer;
};

// ***EXTERNALS*********************************************************************** 

extern "C"
{
    extern TEnvironment  *Environment;
    extern char          *_PrintPrefix;
    extern unsigned long  _DebugMask;
}

// ***PROTOTYPES********************************************************************** 
#ifdef __cplusplus
extern "C" {
#endif
    void
        Initialize(void);
    void
        Print(const char *Format, ...);
    void
        PrintDebug(unsigned long DebugFlag, const char *Format, ...);
#ifdef __cplusplus
}
#endif
// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#include "TEnvironment.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
