/*      file     : common/TLocation.C
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TLocation
                   TRectangle
        RCS      : $Id: TLocation.C,v 1.6 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <math.h>
#include <stdio.h>

#include <TString.h>
#include <TBaseStream.h>
#include <TLocation.h>

/* ***DEFINES************************************************************************* */

#define ParentClass TObject

/* ***FUNCTIONS*********************************************************************** */

TLocation
operator * (double Factor, const TLocation &Loc)

{
    return TLocation(Factor * Loc.x(), Factor * Loc.y());
}

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#include <TLocation.i.C>
#endif

/* =================================================================================== */

TLocation::TLocation(TCoordinate theX, TCoordinate theY)

{
    _X = theX;
    _Y = theY;
}

/* =================================================================================== */

void
TLocation::operator += (const TLocation &aLocation)

{
    _X += aLocation.x();
    _Y += aLocation.y();
}

/* =================================================================================== */

void
TLocation::operator -= (const TLocation &aLocation)

{
    _X -= aLocation.x();
    _Y -= aLocation.y();
}

/* =================================================================================== */

TLocation
TLocation::operator - (const TLocation &aLocation) const

{
    return TLocation( _X - aLocation._X, _Y - aLocation._Y);
}

/* =================================================================================== */

TLocation
TLocation::operator + (const TLocation &aLocation) const

{
    return TLocation( _X + aLocation._X, _Y + aLocation._Y);
}

/* =================================================================================== */

void
TLocation::operator *= (double Factor)

{
    _X *= Factor;
    _Y *= Factor;
}

/* =================================================================================== */

void
TLocation::operator /= (double Divisor)

{
    _X /= Divisor;
    _Y /= Divisor;
}

/* =================================================================================== */

double
TLocation::AngleTo(TLocation &OtherLocation) const

// Compute the angle between the vector going from this location to the OtherLocation
// and the x-axis

{
    /*
  TCoordinate DeltaX;
  TCoordinate DeltaY;
  
  
  DeltaX = OtherLocation.x() - x();
  DeltaY = OtherLocation.y() - y();
  if (DeltaX == 0)
    return (DeltaY < 0)? 3.0 / 2.0 * PI : PI/2.0;
  else
    if (DeltaY == 0)
      return (DeltaX < 0)? PI : 0.0;
    else
    {
        double temp;
        
        temp = atan2((double) DeltaY, (double) DeltaX);
        return (temp < 0)? temp+2*PI : temp;
    }
*/
/*
#ifdef nK
    Warning("atan2 not implemented");
    return -1.0;
#else
*/
    return atan2(OtherLocation.y() - y(), OtherLocation.x() - x());
/*
#endif
*/
}

/* =================================================================================== */

bool
TLocation::Decode(TBaseStream &aStream)

{
    return (aStream.Decode(_X) AND
            aStream.Decode(_Y));
}

/* =================================================================================== */

TCoordinate
TLocation::DistanceTo(const TLocation &aLocation) const

{
    TCoordinate DeltaX;
    TCoordinate DeltaY;

    DeltaX = aLocation.x() - _X;
    DeltaY = aLocation.y() - _Y;
    return sqrt(DeltaX * DeltaX + DeltaY * DeltaY);

}

/* =================================================================================== */

bool
TLocation::Encode(TBaseStream &aStream) const

{
    return (aStream.Encode(_X) AND
            aStream.Encode(_Y));
}

/* =================================================================================== */

TString
TLocation::GetClassName(void) const

{
    return "TLocation";
}

/* =================================================================================== */

TString
TLocation::GetInstanceName(void) const

{
    char line[128];

    sprintf(line, "X:%f, Y:%f", _X, _Y);
    return line;
}

/* =================================================================================== */

double 
TLocation::Length(void) const

{
    return sqrt(_X * _X + _Y * _Y);
}

/* =================================================================================== */

TLocation
TLocation::Normal(void) const

{
    return TLocation(_Y, -_X);
}

/* =================================================================================== */

void
TLocation::Reset(void)

{
    _X = 0.0;
    _Y = 0.0;
}

/* =================================================================================== */

void
TLocation::Set(TCoordinate theX, TCoordinate theY)

{
    _X = theX;
    _Y = theY;
}

/* =================================================================================== */

void
TLocation::SetX(TCoordinate theX)

{
    _X = theX;
}

/* =================================================================================== */

void
TLocation::SetY(TCoordinate theY)

{
    _Y = theY;
}

/* =================================================================================== */

TRectangle::TRectangle(const TLocation Location1, const TLocation Location2)

{
    Locations[0] = Location1;
    Locations[1] = Location2;
}

/* =================================================================================== */

void
TRectangle::operator += (double Frame)

{
    GetMinX() -= Frame;
    GetMaxX() += Frame;
    GetMinY() -= Frame;
    GetMaxY() += Frame;
}

/* =================================================================================== */

void
TRectangle::operator *= (double Factor)

{
    double DeltaX, DeltaY;

    DeltaX = 0.5 * GetDeltaX() * (Factor - 1.0);
    DeltaY = 0.5 * GetDeltaY() * (Factor - 1.0);
    GetMinX() -= DeltaX;
    GetMaxX() += DeltaX;
    GetMinY() -= DeltaY;
    GetMaxY() += DeltaY;
}

/* =================================================================================== */

void
TRectangle::operator += (const TLocation &Vector)

{
    Print("+= (TLocation)");

    Locations[0] += Vector;
    Locations[1] += Vector;
}

/* =================================================================================== */

void
TRectangle::CenterAround(const TLocation &Location)

{
    TCoordinate DeltaX;
    TCoordinate DeltaY;

    DeltaX = 0.5*GetDeltaX();
    DeltaY = 0.5*GetDeltaY();
    Locations[0].SetX(Location.x()-DeltaX);
    Locations[0].SetY(Location.y()-DeltaY);
    Locations[1].SetX(Location.x()+DeltaX);
    Locations[1].SetY(Location.y()+DeltaY);
}

/* =================================================================================== */

bool
TRectangle::Contains(const TLocation &Location) const

{
    return (Location.x() >= GetMinX() AND Location.x() <= GetMaxX() AND
            Location.y() >= GetMinY() AND Location.y() <= GetMaxY());
}

/* =================================================================================== */

bool
TRectangle::Decode(TBaseStream &aStream)

{
    return (Locations[0].Decode(aStream) AND 
            Locations[1].Decode(aStream));
}

/* =================================================================================== */

void
TRectangle::Empty(void)

{
    Locations[0].SetX(MAX_COORDINATE);
    Locations[0].SetY(MAX_COORDINATE);
    Locations[1].SetX(MIN_COORDINATE);
    Locations[1].SetY(MIN_COORDINATE);
}

/* =================================================================================== */

bool
TRectangle::Encode(TBaseStream &aStream) const

{
    return (Locations[0].Encode(aStream) AND 
            Locations[1].Encode(aStream));
}

/* =================================================================================== */

TLocation
TRectangle::Center(void) const

{
    return TLocation(0.5 * (Locations[0].x() + Locations[1].x()),
                     0.5 * (Locations[0].y() + Locations[1].y()));
}

/* =================================================================================== */

TString
TRectangle::GetClassName(void) const

{
    return "TRectangle";
}

/* =================================================================================== */

TCoordinate
TRectangle::GetDeltaX(void) const

{
    return fabs(Locations[0].x() - Locations[1].x());
}

/* =================================================================================== */

TCoordinate
TRectangle::GetDeltaY(void) const

{
    return fabs(Locations[0].y() - Locations[1].y());
}

/* =================================================================================== */

TString
TRectangle::GetInstanceName(void) const

{
    char line[128];

    sprintf(line, "Loc1:(%s) Loc2:(%s)", (char *) Locations[0].GetInstanceName(),
            (char *) Locations[1].GetInstanceName());
    return line;
}

/* =================================================================================== */

void
TRectangle::GetLowerLeftLocation(TLocation &Location) const

{
    Location.SetX(GetMinX());
    Location.SetY(GetMinY());
}

/* =================================================================================== */

TCoordinate &
TRectangle::GetMaxX(void)

{
    return (Locations[0].x() > Locations[1].x())? Locations[0].x() : Locations[1].x();
}

/* =================================================================================== */

TCoordinate
TRectangle::GetMaxX(void) const

{
    return (Locations[0].x() > Locations[1].x())? Locations[0].x() : Locations[1].x();
}

/* =================================================================================== */

TCoordinate &
TRectangle::GetMaxY(void)

{
    return (Locations[0].y() > Locations[1].y())? Locations[0].y() : Locations[1].y();
}

/* =================================================================================== */

TCoordinate
TRectangle::GetMaxY(void) const

{
    return (Locations[0].y() > Locations[1].y())? Locations[0].y() : Locations[1].y();
}

/* =================================================================================== */

TCoordinate &
TRectangle::GetMinX(void)

{
    return (Locations[0].x() <= Locations[1].x())? Locations[0].x() : Locations[1].x();
}

/* =================================================================================== */

TCoordinate
TRectangle::GetMinX(void) const

{
    return (Locations[0].x() <= Locations[1].x())? Locations[0].x() : Locations[1].x();
}

/* =================================================================================== */

TCoordinate &
TRectangle::GetMinY(void)

{
    return (Locations[0].y() <= Locations[1].y())? Locations[0].y() : Locations[1].y();
}

/* =================================================================================== */

TCoordinate
TRectangle::GetMinY(void) const

{
    return (Locations[0].y() <= Locations[1].y())? Locations[0].y() : Locations[1].y();
}

/* =================================================================================== */

void
TRectangle::GetUpperRightLocation(TLocation &Location) const

{
    Location.SetX(GetMaxX());
    Location.SetY(GetMaxY());
}

/* =================================================================================== */

void
TRectangle::Merge(const TLocation &Location)

{
    if (Location.x() < Locations[0].x())
        Locations[0].SetX(Location.x());
    if (Location.y() < Locations[0].y())
        Locations[0].SetY(Location.y());
    if (Location.x() > Locations[1].x())
        Locations[1].SetX(Location.x());
    if (Location.y() > Locations[1].y())
        Locations[1].SetY(Location.y());
}

/* =================================================================================== */

void
TRectangle::Merge(const TRectangle &Rectangle)

{
    TLocation Location;

    Rectangle.GetLowerLeftLocation(Location);
    Merge(Location);
    Rectangle.GetUpperRightLocation(Location);
    Merge(Location);
}

/* =================================================================================== */

bool
TRectangle::Overlaps(const TRectangle &OtherRect) const

{
    return !(GetMinX() > OtherRect.GetMaxX() OR
             GetMinY() > OtherRect.GetMaxY() OR
             GetMaxX() < OtherRect.GetMinX() OR
             GetMaxY() < OtherRect.GetMinY());
}

/* =================================================================================== */
#undef ParentClass
