/*      file     : common/TObjectArray.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TObjectArray.C,v 1.5 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdlib.h>

#include "TObjectArray.h"

// ***LOCAL VARIABLES***************************************************************** 

static int CurrentCompareCode = 0;

// ***FUNCTIONS*********************************************************************** 

int
CompareArrayEntries(const TBaseObject *Entry1, const TBaseObject *Entry2)

{
    return Entry1->Compare((TBaseObject *) Entry2, CurrentCompareCode);
}

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TArray< TBaseObject * >
// =================================================================================== 

TObjectArrayPrimitive::TObjectArrayPrimitive(char *theName, long theMode) :

ParentClass(TARRAY_DEFAULT_INCREMENT, theName)

{
    Mode = theMode;
}

// =================================================================================== 

TObjectArrayPrimitive::TObjectArrayPrimitive(int theIncrement,
                                             char *theName, long theMode) :

ParentClass(theIncrement, theName)

{
    Mode = theMode;
}

/* =================================================================================== */

TObjectArrayPrimitive::~TObjectArrayPrimitive(void)

{
    ActionBeforeDeletion(0, LogicalNrOfItems - 1);
}

/* =================================================================================== */

bool
TObjectArrayPrimitive::ActionBeforeDeletion(long First, long Last)

{
    long         i;
    TBaseObject *Item;

    if (Mode & OA_MODE_AUTOMATIC_DELETION)
        for (i = First; i <= Last; i++)
            if ((Item = Get(i)))
                delete Item;
    return TRUE;
}

/* =================================================================================== */

bool
TObjectArrayPrimitive::ActionAfterInsertion(long First, long Last)

{
    long  i;

    for (i = First; i <= Last; i++)
        if (Mode & OA_MODE_AUTOMATIC_CREATION)
        {
            if ((Get(i) = CreateEntry()) == 0)
                return FALSE;
        }
        else
            Get(i) = 0;
    return TRUE;
}

/* =================================================================================== */

long
TObjectArrayPrimitive::GetNrOfObjects(void) const

{
    return GetNrOfItems();
}

/* =================================================================================== */

void
TObjectArrayPrimitive::SetMode(long theMode)

{
    Mode = theMode;
}

// =================================================================================== 

bool
TObjectArrayPrimitive::Sort(int CompareCode)

{
    CurrentCompareCode = CompareCode;
    qsort((void *) GetItemPtr(), GetNrOfObjects(), sizeof(TBaseObject *),
          (int (*) (const void *, const void *)) CompareArrayEntries);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass



