/*      file     : common/TObjectArray.h
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TObjectArray<>
        RCS      : $Id: TObjectArray.h,v 1.9 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TOBJECTARRAY_H
#define TOBJECTARRAY_H

#include <gcc_pragma_interface.h>

/* ***INCLUDES************************************************************************ */

#include <TBaseObject.h>
#include <TArray.h>

/* ***DEFINES************************************************************************* */

#define OA_MODE_AUTOMATIC_DELETION 1
#define OA_MODE_AUTOMATIC_CREATION 2
#define OA_MODE_AUTOMATIC (OA_MODE_DELETE_OBJECTS | OA_MODE_CREATE_OBJECTS)

/* ***CLASSES************************************************************************* */

class TObjectArrayPrimitive : public TArray<TBaseObject *>

{
public:

    TObjectArrayPrimitive(char *theName, long theMode = 0);
    TObjectArrayPrimitive(int TheIncrement = TARRAY_DEFAULT_INCREMENT,
                          char *theName = "unnamed", long theMode = 0);
    virtual
        ~TObjectArrayPrimitive(void);

    long
        GetNrOfObjects(void) const;
    void
        SetMode(long theMode);
    bool
        Sort(int CompareCode = 0);
    
protected:
    
    virtual TBaseObject *
        CreateEntry(void) const = 0;

    long Mode;

private:

    virtual bool
        ActionBeforeDeletion(long First, long Last);
    virtual bool
        ActionAfterInsertion(long First, long Last);
};

// ===================================================================================

template <class T>
class TObjectArray : public TObjectArrayPrimitive

{
 public:
    
    TObjectArray(char *theName, long theMode = 0);
    TObjectArray(int TheIncrement = TARRAY_DEFAULT_INCREMENT,
                 char *theName = "unnamed", long theMode = 0);
    virtual
        ~TObjectArray(void);

    T * &
        Get(long Index);
    const T *
        Get(long Index) const;

 protected:

    virtual TBaseObject *
        CreateEntry(void) const;
};

/* ***TEMPLATE METHODS**************************************************************** */

#if defined(INCLUDE_TEMPLATES) || defined(TEMPLATEBODIES)
#  include "TObjectArray.t.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
