/*      file     : common/TStatistics.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TStatistics.h,v 1.3 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TSTATISTICS_H
#define TSTATISTICS_H

/* ***INCLUDES************************************************************************ */

#include <TArray.h>
#include <TTimer.h>

/* ***DEFINES************************************************************************* */

#define WrapAround(Value, MaxValue) Value = (Value + 1) % MaxValue
#define EPSILON(Value) (fabs(Value) < 1e-8)

/* ***CLASSES************************************************************************* */

class TStatistics : public TBaseObject

{
 private:
    
    int             NrOfCols;
    TArray<double>  Data;
    long            NrOfElements;
    long            MaxNrOfElements;
    long            InputPointer;
    long            DeletePointer;
    double         *Sums;
    double         *Minima;
    double         *Maxima;
    double         *Deltas;
    double         *Transients;
    int            *Delays;
    int            *DelayInits;

    void
        RemoveElement(int Row = 0);
    void
        UpdateTransient(int Row);

 public:

    TStatistics(long theMaxNrOfElements = -1, int theNrOfCols = 1);
    ~TStatistics(void);

    bool
        AddValue(double Value, int Row = 0);
    TString
        GetClassName(void) const;
    TString
        GetInstanceName(void) const;
    double
        GetMean(int Row = 0);
    double
        GetMinimum(int Row = 0);
    double
        GetMaximum(int Row = 0);
    double
        GetTransientMinimum(int Row = 0);
    void
        SetDelay(int Delay, int Row = 0);
};

// ===================================================================================

class TRealTimeStatistics : public TStatistics

{
public:
    
    TRealTimeStatistics(long theMaxNrOfElements, double thePeriod,
                        int theNrOfCols = 1);
    bool
        AddValue(double Value, int Row = 0);

private:

    TTimer Timer;
    double Interval;
    double Next;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
