/*      file     : common/TTimer.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TTimer.C,v 1.6 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <Basic.h>
#include <Time.h>

#include <TTimer.h>

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#  include "TTimer.i.C"
#endif

/* =================================================================================== */

TTimer::TTimer(TTimer *theParent)

{
    Active = FALSE;
    Parent = theParent;
    CurrentChild = 0;
    Reset();
    StartTime.tv_sec = 0;
    StartTime.tv_usec = 0; /* MR 1-17-94 discovered by PURIFY */
}

/* =================================================================================== */

TTimer::~TTimer(void)

{
    if (Parent)
        CancelChild(this);
}

/* =================================================================================== */

void
TTimer::CancelChild(TTimer *theChild)

{
    if (CurrentChild == theChild)
        CurrentChild = 0;
}

/* =================================================================================== */

double
TTimer::GetElapsedTime(void) const

{
    double ElapsedTime;
    struct timeval Current;

    if (Active)
    {
        GetTime(&Current);
        ElapsedTime = (SumTime.tv_sec + Current.tv_sec - StartTime.tv_sec) +
            ((double) (SumTime.tv_usec + Current.tv_usec - StartTime.tv_usec)) / 1e6;
    }
    else
        return GetStoppedTime();
    return ElapsedTime;
}

/* =================================================================================== */

double
TTimer::GetLap(void)

{
    double ElapsedTime;
    double Difference;

    ElapsedTime = GetElapsedTime();
    Difference = ElapsedTime - LapTime;
    LapTime = ElapsedTime;
    return Difference;
}

/* =================================================================================== */

double
TTimer::GetStoppedTime(void) const

{
    return SumTime.tv_sec + ((double) SumTime.tv_usec) / 1.0e6;
}

/* =================================================================================== */

void
TTimer::Reset(void)

{
    SumTime.tv_sec = 0;
    SumTime.tv_usec = 0;
    LapTime = 0.0;
}

/* =================================================================================== */

bool
TTimer::Start(void)

{
    if (!Active)
    {
        GetTime(&StartTime);
        Active = TRUE;
    }
    return TRUE;
}

/* =================================================================================== */

bool
TTimer::Stop(void)

{
    struct timeval StopTime;

    if (Active)
    {
        GetTime(&StopTime);
        SumTime.tv_sec += StopTime.tv_sec - StartTime.tv_sec;
        SumTime.tv_usec += StopTime.tv_usec - StartTime.tv_usec;
        if (SumTime.tv_usec >= 1000000)
        {
            SumTime.tv_usec -= 1000000;
            SumTime.tv_sec++;
        }
        else
            if (SumTime.tv_usec < 0)
            {
                SumTime.tv_usec += 1000000;
                SumTime.tv_sec--;
            }
        Active = FALSE;
    }
    return TRUE;
}

/* =================================================================================== */

#undef FatherObject
