/*      file     : graphx/grUtil.c
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: grUtil.c,v 1.5 2005/03/25 11:06:10 mr Exp $
*/

#include <graphx.h>

#ifdef DEBUG
char grWhere[128];
#endif

double GrUnitLengths[GrUnitTypes] = {72,2.835};
double GrPaperDimensions[GrUnitTypes][GrPaperTypes][2] = {
  {{8.26875,11.694375},{210.0,297.0}},
  {{8.5,11.0},{215.87302,279.36508}}
};

char *BaseColorNames[GR_MAX_BASE_COLORS] =

{
    "red",
    "springgreen",
    "blue",
    "orange",
    "yellow",
    "brown",
    "violet",
    "cyan"
};

grWindow *grCreateWindow(char *title,int width,int height, int ReservedColors) {
  grWindow *theWindow = (grWindow *)malloc(sizeof(grWindow));
#define RETURN(errorcode) {if (errorcode==0) return theWindow;\
  else {free(theWindow);return NULL;}}
#ifdef DEBUG
  SETFUNC("grWindow *grCreateWindow(char *,int,int)");
  if (width <= 0) FATAL("Trying to open graphics window of non-positive width");
  if (height <= 0) FATAL("Trying to open graphics window of non-positive height");
  if (theWindow == (grWindow *)NULL) FATAL("Unable to allocate grWindow structure");
#endif

  theWindow->noFont = theWindow->topFont = theWindow->theFont = 0;
  theWindow->theFontPS = -1;
  grSetHorizontalTextAlignment(theWindow,GrLeftAlign);
  grSetVerticalTextAlignment(theWindow,GrBottomAlign);
  
  grSetXaxis(theWindow,GrXmin,GrXmax);
  grSetYaxis(theWindow,GrYmin,GrYmax);
  grSetZaxis(theWindow,GrZmin,GrXmax);
  grX11SetSpeed(theWindow,GrFast);
 
  theWindow->MAXX = width-1;
  theWindow->MAXY = height-1;
#ifdef DEBUG
  if ((theWindow->dpy = XOpenDisplay(NULL))==NULL) 
    ERROR("Unable to open display");
#else
  if ((theWindow->dpy = XOpenDisplay(NULL))==NULL) RETURN(1);
#endif
  theWindow->scr = DefaultScreen(theWindow->dpy);
  theWindow->root = DefaultRootWindow(theWindow->dpy);
  theWindow->vis = DefaultVisual(theWindow->dpy,theWindow->scr);
  theWindow->depth = DefaultDepth(theWindow->dpy,theWindow->scr);
  theWindow->gc = DefaultGC(theWindow->dpy,theWindow->scr);
  theWindow->NrOfColorCells = 
    CellsOfScreen(DefaultScreenOfDisplay(theWindow->dpy)) - GR_MAX_BASE_COLORS -
        ReservedColors;

  theWindow->noColors = GR_MAX_COLORS + GR_MAX_BASE_COLORS + ReservedColors;
  theWindow->colors = (XColor *) malloc (theWindow->noColors * sizeof(XColor));

  /*
  fprintf(stderr,"graphx: visual: %ld depth: %ld #colors: %ld\n", 
          (long) theWindow->vis, (long) theWindow->depth, (long) theWindow->noColors);
          */

  theWindow->xsh.flags = (PSize|PMinSize|PMaxSize);
  theWindow->xsh.x = theWindow->xsh.y = 0;
  theWindow->xsh.width = theWindow->xsh.min_width
    = theWindow->xsh.max_width = width;
  theWindow->xsh.height = theWindow->xsh.min_height
    = theWindow->xsh.max_height = height;
  theWindow->win = XCreateSimpleWindow(theWindow->dpy,theWindow->root,
    theWindow->xsh.x,theWindow->xsh.y,theWindow->xsh.width,
    theWindow->xsh.height,1,BlackPixel(theWindow->dpy,theWindow->scr),
    theWindow->colors[theWindow->noColors-1].pixel);
  grX11SelectInput(theWindow,GrStandardEventMask);
  XSetStandardProperties(theWindow->dpy,theWindow->win,title,title,None,
    NULL,0,&(theWindow->xsh));
  
  theWindow->xswa.colormap = XCreateColormap(theWindow->dpy,theWindow->win,
    theWindow->vis,AllocNone);
  
  theWindow->xswa.colormap = DefaultColormap(theWindow->dpy,theWindow->scr);
  theWindow->xswa.bit_gravity = CenterGravity;
  grSetColorMode(theWindow,GrGrey);
  
  XChangeWindowAttributes(theWindow->dpy,theWindow->win,
    (CWColormap|CWBitGravity),&(theWindow->xswa));
  XMapWindow(theWindow->dpy,theWindow->win);

  theWindow->noBackground = 0;  
  grCreateBackground(theWindow,0);
  grSetDrawable(theWindow,0);
  
  grSetLengthUnit(theWindow,GrInch);
  grSetPaperType(theWindow,GrUSletter);
  theWindow->psFile = NULL;
  theWindow->psColor = -1;
  
  RETURN(0);

#undef RETURN
}

void grDestroyWindow(grWindow *theWindow) {
  int i /*, n */;
#ifdef DEBUG
  SETFUNC("void grDestroyWindow(grWindow *)");
#endif

  if (theWindow != 0)
  {
      if (theWindow->psFile != NULL) grClosePostScript(theWindow);
      grCreateBackground(theWindow,0);
      for (i = 0; i < theWindow->MaxColorIndex; i++)
          XFreeColors(theWindow->dpy,theWindow->xswa.colormap,
                      &theWindow->colors[i].pixel, 1, 0);
      free(theWindow->colors);
      free(theWindow->background);
  /*
     this part is still buggy! (MR)
     if (theWindow->noFont > 0) {
     for (n = 0;n < theWindow->topFont;n++) 
     grUnloadFont(theWindow,theWindow->theFonts[n]->family,
     theWindow->theFonts[n]->weight,
     theWindow->theFonts[n]->shape,
     theWindow->theFonts[n]->size);
     for (n = 0;n < theWindow->noFont;n++) free(theWindow->theFonts[n]);
     free(theWindow->theFonts);
     }
     */
      /* XUnmapWindow(theWindow->dpy,theWindow->win);*/
      XWithdrawWindow(theWindow->dpy,theWindow->win,theWindow->scr);
      grX11Flush(theWindow);
      XDestroyWindow(theWindow->dpy,theWindow->win);
  }
}

/*********************** Background Stuff ***************************/

void grCreateBackground(grWindow *theWindow,int nBackgrounds) {
  int n;
#ifdef DEBUG
  SETFUNC("void grCreateBackground(grWindow *,int)");
#endif

  if (nBackgrounds < 0) return;
  for (n = 1;n <= theWindow->noBackground;n++)
    XFreePixmap(theWindow->dpy,theWindow->background[n]);
  if (theWindow->noBackground > 0) free(theWindow->background);
  theWindow->noBackground = nBackgrounds;
  theWindow->background = (Pixmap *)malloc((nBackgrounds+1)*sizeof(Pixmap));
  theWindow->background[0] = theWindow->win;
  for (n = 1;n <= nBackgrounds;n++)
    theWindow->background[n] = XCreatePixmap(theWindow->dpy,theWindow->win,
      theWindow->MAXX+1,theWindow->MAXY+1,theWindow->depth);
}

int grBitBlt(grWindow *theWindow,int dst,int dst_x,int dst_y,int src,
  int src_x,int src_y,int width,int height) {
#ifdef DEBUG
  SETFUNC("int grBitBlt(grWindow *,int,int,int,int,int,int,int,int)");
#endif

  if (src < 0 OR src > theWindow->noBackground) return 0;
  if (dst < 0 OR dst > theWindow->noBackground) return 0;
  if (dst == src) return 0;
  XCopyArea(theWindow->dpy,theWindow->background[src],
    theWindow->background[dst],theWindow->gc,src_x,src_y,width,
    height,dst_x,dst_y);
  return 1;
}

/*********************** Basic Setup Stuff **************************/


int
grAllocateColorTransition(grWindow *theWindow,const char *FromColorName, 
                          const char *ToColorName, int NrOfSteps, 
                          int *BaseOffset)

{
    XColor FromColor;
    XColor ToColor;
    int    i;

    *BaseOffset = theWindow->MaxColorIndex;
    XAllocNamedColor(theWindow->dpy,theWindow->xswa.colormap, FromColorName, 
                     &(theWindow->colors[theWindow->MaxColorIndex]), &FromColor);
    XAllocNamedColor(theWindow->dpy,theWindow->xswa.colormap, ToColorName,
                     &(theWindow->colors[theWindow->MaxColorIndex + NrOfSteps - 1]),
                     &ToColor);
    for (i = 1; i < NrOfSteps - 1; i++)
    {
          theWindow->colors[theWindow->MaxColorIndex + i].red = 
              FromColor.red + (ToColor.red - FromColor.red) * i / NrOfSteps;
          theWindow->colors[theWindow->MaxColorIndex + i].green =
              FromColor.green + (ToColor.green - FromColor.green) * i / NrOfSteps;
          theWindow->colors[theWindow->MaxColorIndex + i].blue = 
              FromColor.blue + (ToColor.blue - FromColor.blue) * i / NrOfSteps;
          XAllocColor(theWindow->dpy,theWindow->xswa.colormap,
                      &(theWindow->colors[theWindow->MaxColorIndex + i]));
    }
    theWindow->MaxColorIndex += NrOfSteps;
    return 1;
}

void grSetColorMode(grWindow *theWindow,int colorMode ) {
  int i, j;
  int value;
  /*  int PowerOfTwo; */
  int EqualColors;
  int NrOfColors;
  XColor DummyColor;
#ifdef DEBUG
  SETFUNC("void grSetColorMode(grWindow *,int)");
  if (colorMode != GrGrey AND colorMode != GrColor)
    ERROR("Trying to set illegal color mode");
#endif

  /*
  if (theWindow->depth > 8)
      fprintf(stderr, "graphx: visual with depth>8 detected.\n");
      */
  /*
  if (colorMode != GrGrey AND colorMode != GrColor) return;
  PowerOfTwo = 0;
  */
  
  NrOfColors = 32;

  /*
  NrOfColors = theWindow->NrOfColorCells;
  while (NrOfColors > 1)
  {
      NrOfColors = NrOfColors >> 1;
      PowerOfTwo++;
  }
  while (PowerOfTwo)
  {
      NrOfColors = NrOfColors << 1;
      PowerOfTwo--;
      
  }
  */
  EqualColors = GR_MAX_COLORS / NrOfColors;
  theWindow->MaxColorIndex = 0;
  /*
  fprintf(stderr, "graphx: using %d colors for greyscale\n", NrOfColors);
  */
  for (i = 0;i < NrOfColors; i++) 
  {
      value = 65535 * i / (NrOfColors - 1);
      for (j = 0; j < EqualColors; j++)
      {
          theWindow->colors[theWindow->MaxColorIndex].red = value;
          theWindow->colors[theWindow->MaxColorIndex].green = value;
          theWindow->colors[theWindow->MaxColorIndex].blue = value;
          XAllocColor(theWindow->dpy,theWindow->xswa.colormap,
                      &(theWindow->colors[theWindow->MaxColorIndex]));
          theWindow->MaxColorIndex++;
      }
  }
  theWindow->BaseColorIndex = theWindow->MaxColorIndex;
  for (i = 0; i < GR_MAX_BASE_COLORS; i++)
  {
      if (XAllocNamedColor(theWindow->dpy,theWindow->xswa.colormap,
                           BaseColorNames[i],
                           &(theWindow->colors[theWindow->MaxColorIndex]),
                           &DummyColor) == 0)
          fprintf(stderr, "graphx: couldn't allocate color '%s'\n", BaseColorNames[i]);
      theWindow->MaxColorIndex++;
  }      
  for (i = 0; i < theWindow->MaxColorIndex;i++) 
    theWindow->colors[i].flags = DoRed|DoGreen|DoBlue;
  if (NrOfColors <= 0)
    fprintf(stderr,"WARNING: grSetColorMode unable to allocate any colors.\n");
}

/************************* PostScript Stuff *************************/

void grOpenPostScript(grWindow *theWindow,char *postScriptFileName) {
#ifdef DEBUG
  SETFUNC("void grOpenPostScript(grWindow *,char *)");
  if (theWindow->psFile != NULL) WARNING("Trying to open PostScript screen"
    " that is already open. Closing it.");
#endif
  if (theWindow->psFile != NULL) grClosePostScript(theWindow);
  
  theWindow->psFile = fopen(postScriptFileName,"w");
#ifdef DEBUG
  if (theWindow->psFile == NULL) FATAL("Unable to open PostScript screen");
#endif
  if (theWindow->speed == GrSlow) setbuf(theWindow->psFile,NULL);
  fprintf(theWindow->psFile,"%%!PS-Adobe-2.0\n");
  fprintf(theWindow->psFile,"%%%%Creator: graphx toolbox\n");
  fprintf(theWindow->psFile,"%%%%BoundingBox: 0 0 %g %g\n",
    PSLENX(theWindow),PSLENY(theWindow));
  fprintf(theWindow->psFile,"/m {moveto} def\n");
  fprintf(theWindow->psFile,"/l {lineto} def\n");
  fprintf(theWindow->psFile,"/s {stroke} def\n");
  fprintf(theWindow->psFile,"/w {setlinewidth} def\n");
  fprintf(theWindow->psFile,"/b {newpath moveto 1 0 rlineto 0 1 rlineto"
          " -1 0 rlineto closepath fill} def\n");
  strcpy(theWindow->psFileName, postScriptFileName);
}

void grClosePostScript(grWindow *theWindow) {
#ifdef DEBUG
  SETFUNC("void grClosePostScript(grWindow *)");
  if (theWindow->psFile == NULL) WARNING("Trying to close unopened PostScript screen");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"showpage");
  fclose(theWindow->psFile);
  theWindow->psFile = NULL;
}

void grSetLengthUnit(grWindow *theWindow,int unit) {
#ifdef DEBUG
  SETFUNC("void grSetLengthUnit(grWindow *,int)");
  if (unit != GrInch AND unit != GrMm)
    ERROR("Trying to set illegal length unit");
#endif

  theWindow->psLengthUnit = unit;
}

void grSetPaperType(grWindow *theWindow,int type) {
#ifdef DEBUG
  SETFUNC("void grSetPaperType((grWindow *,int)");
  if (type != GrA4 AND type != GrUSletter) 
    ERROR("Trying to set illegal paper type");
#endif

  theWindow->psPaperType = type;
}
