/*      file     : parbalance/TBaseBalancer.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TBaseBalancer.C,v 1.7 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TDebug.h>
#include <TConfiguration.h>
#include <TExtendedCPNInfo.h>
#include <TNetwork.h>

#include "ParBalanceConfig.h"
#include "TTransferLoadInfo.h"

#include "TBaseBalancer.h"

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TBaseBalancer::TBaseBalancer(void)

{
    LocalTime = (double) INVALID;
    LocalLoad = (double) INVALID;
    LocalPerf = (double) INVALID;
    Configuration->SetDefault(TConfigEntry(BALANCER_GROUP, BALANCER_NAME_SAMPLINGINTERVAL,
                                           BALANCER_DEFAULT_SAMPLINGINTERVAL));
    Configuration->SetDefault(TConfigEntry(BALANCER_GROUP, BALANCER_NAME_DAMPENING,
                                           BALANCER_DEFAULT_DAMPENING));
    Configuration->SetDefault(TConfigEntry(BALANCER_GROUP, BALANCER_NAME_BLOCK,
                                           BALANCER_DEFAULT_BLOCK));
}

// =================================================================================== 

bool
TBaseBalancer::ExecuteSubTimeStep(TTimeStep &)

{
    return TRUE;
}

// =================================================================================== 

bool
TBaseBalancer::Run(void)

{
    Print(DM_LIFESPAN, "Run");
    Interval = (int) Configuration->GetLong(BALANCER_GROUP, 
                                            BALANCER_NAME_SAMPLINGINTERVAL);
    Block = (bool) Configuration->GetLong(BALANCER_GROUP, BALANCER_NAME_BLOCK);
    Dampening = Configuration->GetDouble(BALANCER_GROUP, BALANCER_NAME_DAMPENING);
    if (Interval <= 0)
    {
        Error("Run: invalid load-balancing interval %ld", Interval);
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TBaseBalancer::SetTransferInTimeUnits(TCPNID Destination, TTimeStep &TimeStep, 
                                      double MyTime, double MyPerf,
                                      double OtherTime, double OtherPerf)

{
    double TransferLoad;

    TransferLoad = OtherPerf * MyPerf * (MyTime - OtherTime) / (MyPerf + OtherPerf);
    
    if (!SetTransfer(Destination, TimeStep, TransferLoad))
    {
        Error("SetTransferInTimeUnits: cannot set transfer load to %lX", Destination);
        return FALSE;
    }    
    return TRUE;
}

// =================================================================================== 

bool
TBaseBalancer::SetTransfer(TCPNID Destination, TTimeStep &TimeStep, double TransferLoad)

{
    TExtendedCPNInfo  *CPNInfo;
    TTransferLoadInfo *LoadInfo;
    TInfoID            InfoID;
    
    if ((CPNInfo = Network->GetExtendedCPNInfo(Destination, TRUE)) == 0)
    {
        Error("SetTransferLoad: cannot retrieve ExtendedCPNInfo");
        return FALSE;
    }
    InfoID = TTransferLoadInfo::ClassInfo->GetInfoID();
    if ((LoadInfo = (TTransferLoadInfo *) CPNInfo->GetInfo(InfoID, TRUE)) == 0)
    {
        Error("SetTransferLoad: cannot retrieve TTransferLoadInfo");
        return FALSE;
    }
    
    LoadInfo->Set(TimeStep, Dampening * TransferLoad);
    
    Print(DM_LOAD, "SetTransferLoad: %s", (char *) LoadInfo->GetInstanceName());
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
