/*      file     : parcontrol/TApplicationSlave.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TApplicationSlave.h,v 1.14 2005/03/25 11:07:02 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TAPPLICATIONSLAVE_H
#define TAPPLICATIONSLAVE_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>

#include <TMessageRecipient.h>
#include <TObjectHandler.h>
#include <TStatHandler.h>
#include <TTimeStep.h>
#include <TDuration.h>

// ***FORWARDS************************************************************************

class TEnvironment;
class TCommunication;
class TBaseDataManager;
class TAppEventHandler;
class TObjectManager;
class TNetwork;
class TParallel;
class TSamplingDescription;
class TStatManager;
class TSynchronize;
class TDynamicBalancer;
class TTimer;

#include <TContainerObject.forward.h>
#include <TCPNInfo.forward.h>
#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***CLASSES************************************************************************* 

/** Class handling application control for a slave (load balancing, data storage).
  */

class TApplicationSlave : public TBaseObject, public TMessageRecipient, 
                          public TObjectHandler, public TStatHandler

{
public:

    TApplicationSlave(int argc, char *argv[]);
    virtual ~TApplicationSlave(void);

    virtual bool
    Activate(void);

    bool
    AddApplicationNodes(TNodeList *Nodes);

    bool
    AddApplicationEdges(TEdgeList *Edges);

    bool
    CleanUpAfterTransfer(void);

    virtual bool
    Run(void);

    bool
    MigrateContainerObjects(const TCPNInfo *Destination, TContainerObjectList *List);

protected:

    virtual TString
    GetClassName(void) const;

    virtual TString
    GetInstanceName(void) const;

    virtual bool
    ExecuteSubTimeStep(void);

    virtual bool
    HandleAction(TParallelObject *Object, TObjectAction Action);

    virtual bool
    HandleEvent(TEventID EventID, TBaseMessage *aMessage, int &Handled);

    virtual bool
    IdleTime(void);

    virtual bool
    InitializeDataAfterReception(void);

    TTimeStep  ApplicationTimeStep;
    TDuration  ApplicationSequence;
    TNodeList  ApplicationNodes;
    TEdgeList  ApplicationEdges;
    TEdgeList  BoundaryEdges;
    TEdgeList  NonBoundaryEdges;
    TNodeList  InitNodes;
    TEdgeList  InitEdges;
    TTimer    *ApplicationTimer;
    TTimer    *StatTimer;
    TTimer    *NodeTimer;
    TTimer    *EdgeTimer;

    TDynamicBalancer   *DynamicBalancer;

private:

    bool
    ActivateAppEventHandler(void);

    bool
    ActivateStatistics(void);

    virtual bool
    Collect(TSamplingDescription *Description);

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);
    bool
    HandleEventInitializeDataAfterReception(void);

    bool
    HandleEventStartApplicationSequence(TBaseMessage *Message);

    bool
    HandleMapStatResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    bool
    SynchronizeApplications(void);
    
    TEnvironment       *MyEnvironment;
    TCommunication     *MyCommunication;
    TParallel          *MyParallel;
    TNetwork           *MyNetwork;
    TBaseDataManager   *MyBaseDataManager;
    TObjectManager     *MyObjectManager;
    TStatManager       *MyStatManager;

    TSynchronize       *AppSync;
    bool                SlaveApplication;
    bool                MasterApplication;

    TCPNInfoList        TransferDestinations;
    int                 RemainingTimeStepsInSequence;

    TSamplingDescription  *MapStatSamplingDescription;
    TAggregationInfo      *MapStatAggregationInfo;
    long                   MapStatInterval;

    TAppEventHandler *AppEventHandler;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
