/*      file     : TMapStat.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMapStat.C,v 1.2 2005/03/25 11:07:02 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>
#include <TLocationInfo.h>
#include <TNode.h>

#include "TMapStat.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TMapStat::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TMapStat.i.C"
#endif

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TMapStat::TMapStat(TClassInfo *theClassInfo, TParallelID theParallelID) :

ParentClass(theClassInfo, theParallelID)

{
    Reset();
}

/* =================================================================================== */

TMapStat::TMapStat(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Reset();
}

/* =================================================================================== */

TMapStat::~TMapStat(void)

{
}

/* =================================================================================== */

bool
TMapStat::Accumulate(const TStatObject *StatObject)

{
    const TMapStat *Stat = (const TMapStat *) StatObject;

    return ParentClass::Accumulate(StatObject);
}

/* =================================================================================== */

bool
TMapStat::Collect(TNodeList *Nodes, TEdgeList *Edges)

{
    TNodeEnum CNode;
    TInfoID   LocInfoID;

    LocInfoID = TLocationInfo::ClassInfo->GetInfoID();
    MapRectangle.Empty();
    CNode.WalksForwardThrough(Nodes);
    while (CNode())
    {
        MapRectangle.Merge(CNode->Loc(LocInfoID));
        CNode++;
    }
    NrOfNodes = Nodes->GetNrOfElements();
    NrOfEdges = Edges->GetNrOfElements();
    return TRUE;
}

/* =================================================================================== */

bool
TMapStat::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(aStream, TransferInfo) AND
            aStream->Decode(NrOfNodes, TMAPSTAT_INT) AND
            MapRectangle.Decode(*aStream));
}

/* =================================================================================== */

bool
TMapStat::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(aStream, TransferInfo) AND
            aStream->Encode(NrOfNodes, TMAPSTAT_INT) AND
            MapRectangle.Encode(*aStream));
}

/* =================================================================================== */

TString
TMapStat::GetClassName(void) const

{
    return "TMapStat";
}

/* =================================================================================== */

void
TMapStat::Reset(void)

{
    MapRectangle.Empty();
}

// =================================================================================== 
#undef ParentClass
