/*      file     : parbalance/TMapStat.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMapStat.h,v 1.2 2005/03/25 11:07:02 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMAPSTAT_H
#define TMAPSTAT_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TLocation.h>
#include <TDLList.h>
#include <TStatObject.h>

// ***FORWARDS************************************************************************ 

#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***CLASSES************************************************************************* 

class TMapStat : public TStatObject

{
public:

    TMapStat(TClassInfo *theClassInfo, TParallelID theParallelID = INVALID);
    TMapStat(TParallelID theParallelID = INVALID);
    virtual ~TMapStat(void);

    bool
    Collect(TNodeList *Nodes, TEdgeList *Edges);

    inline TRectangle &
    GetMapRectangle(void);

    bool
    Write(const TTimeStep &BaseTimeStep, TBaseStream *Stream);

    static bool
    WriteStatDescription(TBaseStream *Stream);

    static TClassInfo *ClassInfo;

protected:

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

private:

    virtual bool
    Accumulate(const TStatObject *StatObject);

    virtual void
    Reset(void);

private:

    TRectangle MapRectangle;
    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define TMAPSTAT_INT 2
    int        NrOfNodes;
    int        NrOfEdges;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ***TYPES***************************************************************************

typedef TDLList < TMapStat > TMapStatList;
typedef TDLListEnum < TMapStat > TMapStatEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TMapStat.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
