/*      file     : parstat/TStatObject.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TStatObject.C,v 1.3 2005/03/25 11:08:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

/* ***INCLUDES************************************************************************ */

#include <stdio.h>
#include <math.h>

#include <TBaseStream.h>

#include "TStatObject.h"

/* ***DEFINES************************************************************************* */

#define ParentClass TParallelListObject

/* ***FUNCTIONS*********************************************************************** */

double
SecureDiv(double a, double b)

{
    if (fabs(b) < 1.e-10)
        return -1.0;
    else
        return a / b;
}

/* ***METHODS************************************************************************* */

#ifdef NO_INLINE
#  include <TStatObject.i.C>
#endif

/* =================================================================================== */

TStatObject::TStatObject(TClassInfo *theClassInfo, TParallelID theParallelID) :

ParentClass(theClassInfo, theParallelID)

{
    Reset();
}

/* =================================================================================== */

bool
TStatObject::Accumulate(const TStatObject *StatObject)

{
    NrOfSamples += StatObject->NrOfSamples;
    Fraction += StatObject->Fraction;
    return TRUE;
}

/* =================================================================================== */

bool
TStatObject::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    return (ParentClass::Decode(aStream, TransferInfo) AND
            TimeStep.Decode(aStream) AND
            aStream->Decode(NrOfSamples) AND
            aStream->Decode(Fraction));
}

/* =================================================================================== */

bool
TStatObject::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    return (ParentClass::Encode(aStream, TransferInfo) AND
            TimeStep.Encode(aStream) AND
            aStream->Encode(NrOfSamples) AND
            aStream->Encode(Fraction));
}

/* =================================================================================== */

bool
TStatObject::Evaluate(void)

{
    return TRUE;
}

/* =================================================================================== */

TString
TStatObject::GetClassName(void) const

{
    return "TStatObject";
}

/* =================================================================================== */

TString
TStatObject::GetInstanceName(void) const

{
    char Line[128];

    sprintf(Line, "(%s) TS %s Smpls:%ld Fract:%f",
            (char *) ParentClass::GetInstanceName(),
            (char *) TimeStep.GetInstanceName(), NrOfSamples, Fraction);
    return Line;
}

/* =================================================================================== */

void
TStatObject::Reset(void)

{
    NrOfSamples = 1;
    Fraction = 1.0;
}

/* =================================================================================== */
#undef ParentClass

