/*      file     : partools/TAdjacenceInfo.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TAdjacenceInfo.C,v 1.7 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TBaseStream.h>

#include <TNode.h>

#include <TTransferInfo.h>
#include <TClassInfo.h>
#include <TObjectManager.h>

#include "TAdjacenceInfo.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TAdjacenceInfo::ClassInfo = 0;

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#  include "TAdjacenceInfo.i.C"
#endif

// =================================================================================== 
#define ParentClass TParallelObject
// =================================================================================== 

TAdjacenceInfo::TAdjacenceInfo(TClassInfo *theClassInfo, TParallelID theParallelID) :

ParentClass(theClassInfo, theParallelID)

{
    NodeClassID = INVALID;
    Length = (double) INVALID;
}

// =================================================================================== 

bool
TAdjacenceInfo::ComputeEuclideanLength(TInfoID LocationInfoID)

{
    TNode *FromNode = 0, *ToNode = 0;

    if ((FromNode = GetFromNode(TRUE)) == 0 OR
        (ToNode = GetToNode(TRUE)) == 0)
    {
        Warning("GetEuclideanLength: cannot retrieve nodes");
        return FALSE;
    }
    Length = FromNode->Loc(LocationInfoID).DistanceTo(ToNode->Loc(LocationInfoID));
    return TRUE;
}

// =================================================================================== 

bool
TAdjacenceInfo::Decode(TBaseStream *Stream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)
        return (Stream->Decode(NodeClassID) AND
                Stream->Decode(FromNodeID, 2) AND
                Stream->Decode(Length));
    else
        return TRUE;
}

// =================================================================================== 

bool
TAdjacenceInfo::Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(Stream, TransferInfo))
        return FALSE;
    if (TransferInfo->TransferMode == TM_ConstData)     
        return (Stream->Encode(NodeClassID) AND
                Stream->Encode(FromNodeID, 2) AND
                Stream->Encode(Length));
    else
        return TRUE;    
}

// =================================================================================== 

TString
TAdjacenceInfo::GetClassName(void) const

{
    return "TAdjacenceInfo";
}

// =================================================================================== 

TNode *
TAdjacenceInfo::GetFromNode(bool Warn) const

{
    TClassInfo *NodeClassInfo;
    TNode      *FromNode;

    if (NodeClassID == INVALID)
    {
        Error("invalid node class ID");
        return 0;
    }
    NodeClassInfo = ObjectManager->GetClassInfo(NodeClassID);
    if ((FromNode = (TNode *) NodeClassInfo->GetObjectPtr(FromNodeID, Warn)) == 0)
    {
        if (Warn)
            Warning("cannot retrieve pointer FromNode '%s' for ParallelID %ld",
                    (char *) NodeClassInfo->GetInstancePtr(), FromNodeID);
        return 0;
    }
    return FromNode;
}

// =================================================================================== 

TString
TAdjacenceInfo::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "(%s) From:%ld To:%ld L:%f", (char *) ParentClass::GetInstanceName(),
            FromNodeID, ToNodeID, Length);
    return Line;
}

// =================================================================================== 

double
TAdjacenceInfo::GetLength(TInfoID LocationInfoID)

{
    if (Length == (double) INVALID)
        if (LocationInfoID == INVALID)
            Warning("GetLength: no location info available");
        else
            if (!ComputeEuclideanLength(LocationInfoID))
                Warning("GetLength: could not compute Euclidean length");
    return Length;
}

// =================================================================================== 

TNode *
TAdjacenceInfo::GetToNode(bool Warn) const

{
    TClassInfo *NodeClassInfo;
    TNode      *ToNode;

    if (NodeClassID == INVALID)
    {
        Error("invalid node class ID");
        return 0;
    }
    NodeClassInfo= ObjectManager->GetClassInfo(NodeClassID);
    if ((ToNode = (TNode *) NodeClassInfo->GetObjectPtr(ToNodeID, Warn)) == 0)
    {
        if (Warn)
            Warning("cannot retrieve pointer ToNode '%s' for ParallelID %ld",
                    (char *) NodeClassInfo->GetInstancePtr(), ToNodeID);
        return 0;
    }
    return ToNode;
}

// =================================================================================== 

bool
TAdjacenceInfo::IsDistributable(void) const

{
    return (Flags & AI_FLAG_DISTRIBUTABLE) != 0;
}

// =================================================================================== 
#undef ParentClass
