/*      file     : partools/TAdjacenceInfo.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TAdjacenceInfo.h,v 1.8 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TADJACENCE_H
#define TADJACENCE_H

// ***INCLUDES************************************************************************ 

#include <TParallelObject.h>

// ***FORWARDS************************************************************************

class TClassInfo;
class TNode;

// ***DEFINES************************************************************************* 

#define AI_FLAG_DISTRIBUTABLE (PO_FLAG_FIRST_FREE_FLAG)

// ***CLASSES************************************************************************* 

class TAdjacenceInfo : public TParallelObject

{
public:
    
    TAdjacenceInfo(TClassInfo *theClassID, TParallelID theParallelID);

    TNode *
    GetFromNode(bool Warn = TRUE) const;

    inline TParallelID
    GetFromNodeID(void) const;

    double
    GetLength(TInfoID LocationInfoID);

    TNode *
    GetToNode(bool Warn = TRUE) const;

    inline TParallelID
    GetToNodeID(void) const;

    inline void
    MarkAsDistributable(void);

    inline void
    SetFromNodeID(TParallelID theFromNodeID);

    inline void
    SetLength(double theLength);

    inline void
    SetNodeClassID(TClassID theNodeClassID);
    
    inline void
    SetToNodeID(TParallelID theToNodeID);

    inline void
    SetWeight(double theWeight);
    
static TClassInfo *ClassInfo;
    
private:

    bool
    ComputeEuclideanLength(TInfoID LocationInfoID);

    virtual bool
    Decode(TBaseStream *Stream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *Stream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual TString
    GetInstanceName(void) const;

    virtual bool
    IsDistributable(void) const;

    /** Original Node IDs. */

    TClassID NodeClassID;

    //  <<< DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    TParallelID  FromNodeID;
    TParallelID  ToNodeID;
    double       Length;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
        
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TAdjacenceInfo.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
