/*      file     : partools/TClassInfo.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TClassInfo.C,v 1.7 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TParallelObject.h>

#include "TClassInfo.h"

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#include "TClassInfo.i.C"
#endif

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TClassInfo::TClassInfo(const char *theClassname)

{
    Classname = theClassname;
    InfoClasses = 0;
    Table = 0;
}

// =================================================================================== 

TClassInfo::TClassInfo(int theFlags, const char *theContainerClassName, 
                       const char *theClassname, 
                       TObjectHandler *theObjectHandler)

{
    ClassFlags = theFlags;
    ClassID = INVALID;
    InfoID = INVALID;
    ContainerClass = 0;
    ContainerClassname = theContainerClassName;
    Classname = theClassname;
    ObjectHandler = theObjectHandler;
    NrOfInfoClasses = 0;
    InfoClasses = 0;
    Table = 0;
    if (HasTable())
        Table = new TParallelObjectTable;
}

// =================================================================================== 

TClassInfo::~TClassInfo(void)

{
    DeleteObject(Table);
    DeleteObjectArray(InfoClasses);
}

// =================================================================================== 

int
TClassInfo::Compare(TBaseObject *OtherObject, int CompareCode) const

{
    TClassInfo *OtherInfo = (TClassInfo *) OtherObject;

    return StrCmp(Classname, OtherInfo->GetName(), CompareCode);
}

// =================================================================================== 

void
TClassInfo::Delete(const TParallelObject *Object) const

{
    TIDContainerEntry < TParallelObject > *Entry;

    if (Table AND Object->GetParallelID() != INVALID)
    {
        // debug(Print(DM_CLASS, "Delete PID %ld", Object->GetParallelID()));
        if ((Entry = Table->Delete(Object->GetParallelID())) == 0)
            Warning("Delete: ParallelID %ld not found", Object->GetParallelID());
        else
        {
            //      Print("delete ParallelID %ld", Object->GetParallelID());
            delete Entry;
        }
    }
}

// =================================================================================== 

TString
TClassInfo::GetClassName(void) const

{
    return "TClassInfo";
}

// =================================================================================== 

TString
TClassInfo::GetInstanceName(void) const

{
    char Line[256];

    if (IsContainer())
        sprintf(Line, "Container:%s ClassID:%d", Classname(), ClassID);
    else
        sprintf(Line, "Info:%s ClassID:%d InfoID:%d %s(%s)",
                    Classname(), ClassID, InfoID, 
                HasTable()? "table ":"no table ",
                ContainerClass? ((char *) ContainerClass->GetInstanceName()):"no container");
    return Line;
}

// =================================================================================== 

void
TClassInfo::Insert(const TParallelObject *Object) const

{
    if (Table AND Object->GetParallelID() !=  INVALID)
    {
        TIDContainerEntry <TParallelObject > *NewEntry = 
            new TIDContainerEntry < TParallelObject > (Object->GetParallelID(), Object);
        // debug(Print(DM_CLASS, "Insert PID %ld", Object->GetParallelID()));
        if (!Table->Insert(NewEntry, TRUE))
            Warning("Insert: ParallelID %ld is not unique!", Object->GetParallelID());
        //      Print("insert ParallelID %ld", Object->GetParallelID());
    }
}

// =================================================================================== 
#undef ParentClass
