/*      file     : partools/TTimeStep.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TTimeStep.h,v 1.7 2005/03/25 11:08:15 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TTIMESTEP_H
#define TTIMESTEP_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>

// ***FORWARDS************************************************************************

class TBaseStream;

// ***CLASSES************************************************************************* 

/** Utility class for defining a point of time within the simulation. 
  */


class TTimeStep : public TBaseObject

{
 public:

    TTimeStep(long theTimeStep = 0, short int theSubTimeStep = 0);

    inline 
    operator long (void) const;

    inline long
    operator = (long theTimeStep);

    long
    operator ++ (int);

    inline long
    operator - (const TTimeStep &aTimeStep) const;

    inline TTimeStep
    operator + (long Delta) const;

    inline void
    operator += (long Delta);

    inline bool
    operator == (const TTimeStep &aTimeStep) const;

    inline bool
    operator > (const TTimeStep &aTimeStep) const;

    inline bool
    operator < (const TTimeStep &aTimeStep) const;

    inline long
    GetLinear(void) const;

    static int
    GetNrOfSubTimeSteps(void);

    inline int
    GetSubStep(void) const;

    static void
    SetNrOfSubTimeSteps(int NrOfSubSteps);

    bool
    Decode(TBaseStream *aStream);

    bool
    Encode(TBaseStream *aStream) const;

    virtual TString
    GetInstanceName(void) const;

    inline int
    Hours(void) const;

    inline int
    Minutes(void) const;

 private:

    virtual TString
    GetClassName(void) const;

    long              TimeStep;
    short int         SubTimeStep;

    static short int  NrOfSubTimeSteps;
};

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TTimeStep.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
