/*      file     : pvm/TPVMMessage.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TPVMMessage.C,v 1.13 2005/03/25 11:08:26 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <string.h>

#include "Pvm.h"
#include "TDestination.h"
#include "TPVMMessage.h"

// ***METHODS************************************************************************* 

#include "TPVMMessage.Bodies.C"

// =================================================================================== 
#define ParentClass TBaseMessage
// =================================================================================== 

TPVMMessage::TPVMMessage(TCommunicationControl *theControl, 
                         int theBufferID, TCPNID theSender) :

ParentClass(theControl, theSender)

{
    _Init();
    BufferID = theBufferID;
}

// =================================================================================== 

TPVMMessage::TPVMMessage(TCommunicationControl* theControl, TMessageID theID, 
                         TCPNID theSender, TDestination *theDestination, 
                         TMessageTag theTag, TMessageType theType, 
                         TMessageFlags theFlags, int theEncoding) :

                         TBaseMessage(theControl, theID, theSender, theDestination,
                                      theTag, theType, theFlags)

{
    _Init();
    Encoding = theEncoding;
}

// =================================================================================== 

TPVMMessage::~TPVMMessage(void)

{
    int Status;

    if (BufferID != INVALID_ID AND WasSaved)
    {
        if ((Status = pvm_freebuf(BufferID)) < 0)
            Error("DESTRUCTOR: pvm_freebuf returns %d", Status);
        BufferID = INVALID_ID;
    }
}

// =================================================================================== 

bool
TPVMMessage::BeginSend(void)

{
    if ((BufferID = pvm_initsend(Encoding)) < 0)
    {
        Error("BeginSend: pvm_sendinit returns %d", BufferID);
        return FALSE;
    }
    return ParentClass::BeginSend();
}

// =================================================================================== 

bool
TPVMMessage::Deselect(void)

{
    int OldBufferID;

    if ((OldBufferID = pvm_setrbuf(0)) < 0)
    {
        Error("Deselect: pvm_setrbuf returns %d", OldBufferID);
        return FALSE;
    }    
    WasSaved = TRUE;
    debug(Print(DM_MSG_CONTROL, "Deselect: old receive buffer %d", OldBufferID));
    return TRUE;
}

// =================================================================================== 

bool
TPVMMessage::EndSend(void)

{
    int           PvmError;
    const TCPNID *Recipients = 0;
    int          *IntRecipients = 0;

    if (!ParentClass::EndSend())
        return FALSE;
    switch (GetDestination()->GetRecipient())
    {
    case DEST_LIST:
    case DEST_ALL:
    case DEST_ALL_BUT_MYSELF:
        if ((Recipients = GetDestination()->GetRecipients()) == 0)
        {
            Error("cannot retrieve list of recipients");
            return FALSE;
        }
        if (sizeof(TCPNID) != sizeof(int))
        {
            int i;

            IntRecipients = new int [GetDestination()->GetNrOfRecipients()];
            for (i = 0; i < GetDestination()->GetNrOfRecipients(); i++)
                IntRecipients[i] = (int) Recipients[i];
            if ((PvmError = 
                 pvm_mcast((int *) IntRecipients, (int) GetDestination()->GetNrOfRecipients(),
                           PVM_DEFAULT_MSGTYPE)) < 0)
            {
                Error("EndMessage: pvm_cast returns %d", PvmError);
                return FALSE;
            }
            DeleteObjectArray(IntRecipients);
        }
        else        
        {
            debug(Print(DM_MSG, "EndSend: pvm_mcast(*, %d, %d)", 
                        GetDestination()->GetNrOfRecipients(), PVM_DEFAULT_MSGTYPE));
            if ((PvmError = 
                 pvm_mcast((int *) Recipients, (int) GetDestination()->GetNrOfRecipients(),
                           PVM_DEFAULT_MSGTYPE)) < 0)
            {
                Error("EndMessage: pvm_cast returns %d", PvmError);
                return FALSE;
            }
        }
        if (GetDestination()->GetFlags() & DEST_FLAG_INCLUDES_MYSELF)
        {
            debug(Print(DM_MSG, "EndSend: pvm_send(%d, %d)", 
                        (int) GetSender(), PVM_DEFAULT_MSGTYPE));
            if ((PvmError = pvm_send((int) GetSender(), PVM_DEFAULT_MSGTYPE)) < 0)
            {
                Error("EndMessage: pvm_send returns %d", PvmError);
                return FALSE;
            }
        }
        break;
        
    default:
        debug(Print(DM_MSG, "EndSend: pvm_send(%d, %d)", 
                    (int) GetDestination()->GetRecipient(), PVM_DEFAULT_MSGTYPE));
        if ((PvmError = pvm_send((int) GetDestination()->GetRecipient(), 
                                 PVM_DEFAULT_MSGTYPE)) < 0)
        {
            Error("EndMessage: pvm_send returns %d", PvmError);
            return FALSE;
        }
    }
    return TRUE;
}

// =================================================================================== 

TString
TPVMMessage::GetClassName(void) const

{
    return "TPVMMessage";
}

// =================================================================================== 

TString
TPVMMessage::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "%s BufID:%d", (char *) ParentClass::GetInstanceName(), BufferID);
    return Line;
}

// =================================================================================== 

void
TPVMMessage::_Init(void)

{
    WasSaved = FALSE;
    BufferID = INVALID_ID;
    Encoding = INVALID;
}

// =================================================================================== 

bool
TPVMMessage::Select(void)

{
    int OldBufferID;

    if (BufferID == INVALID)
    {
        Error("Select: invalid buffer ID");
        return FALSE;
    }
    if (GetState() == MSG_STATE_SENDING)
    {
        if ((OldBufferID = pvm_setsbuf(BufferID)) < 0)
        {
            Error("Select: pvm_setsbuf returns %d", OldBufferID);
            return FALSE;
        }
        debug(Print(DM_MSG_CONTROL, "Select: send buffer %d -> %d", OldBufferID, BufferID));
    }
    else
        if (GetState() == MSG_STATE_RECEIVING)
        {
            if ((OldBufferID = pvm_setrbuf(BufferID)) < 0)
            {
                Error("Select: pvm_setrbuf returns %d", OldBufferID);
                return FALSE;
            }    
            debug(Print(DM_MSG_CONTROL, "Select: receive buffer %d -> %d", 
                        OldBufferID, BufferID));
        }
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
